--------------------------------------------------------------------------------
-- Broker_ProfessionsMenu v1.6                                                --
-- Author: Sanori/Pathur                                                      --
-- Changed: 27.07.10                                                          --
--------------------------------------------------------------------------------



--------------------------------------------------------------------------------
-- Register Events, Init Variables                                            --
--------------------------------------------------------------------------------
local _, me = ...                                 --Includes all functions and variables

me.frame = CreateFrame("FRAME")
me.frame:RegisterEvent("ADDON_LOADED")
me.frame:RegisterEvent("PLAYER_LOGIN")
me.frame:RegisterEvent("PLAYER_LOGOUT")
me.frame:RegisterEvent("PLAYER_REGEN_DISABLED")
me.frame:RegisterEvent("PLAYER_REGEN_ENABLED")
me.frame:RegisterEvent("TRADE_SKILL_UPDATE")
me.frame:RegisterEvent("TRADE_SKILL_SHOW")

me.dewdrop = LibStub('Dewdrop-2.0', true)      --Dropdownmenu
me.TipHooker = LibStub("LibTipHooker-1.1")     --Tooltip mod

me.launcher = {}                                  --Quick Launcher
me.sharedcds = {}                                 --shared cds

if me.P.SharedCDs then
 for k, v in pairs(me.P.SharedCDs) do
  if GetSpellInfo(k) then                         --check if spell exists
   me.sharedcds[GetSpellInfo(k)] = v
  end
 end
end



--------------------------------------------------------------------------------
-- GET SKILLS, WEAPON SKILLS, RANKS, ICONS                                    --
--------------------------------------------------------------------------------
--Localize Profession / Include Icons / Set Secure or Disabled
function me:LocalizeProf(name, id)
  local spell = {}
  spell.name,_,spell.icon,_,_,_,_,_,_ = GetSpellInfo(id)
   if (strfind(name,",Secure")~=nil) then
    spell.secure = true
   elseif (strfind(name,",Disabled")~=nil) then
    spell.disabled = true
   end
  spell.id = id
  return spell
end
--Localize List of all Professions in idarray and Creat Lines
function me:LocalizeList(profarray)               --Professions ID-Array from professions.lua
  local k, v, kk, vv
  local result = {}
  local spell = {}
  for k, v in pairs(profarray) do
   if (type(v)=="number") then
    result = me:LocalizeProf(k,v)
    spell[result.name] = result
   else
    result = me:LocalizeProf(k,v["*"])
    spell[result.name] = result
    local help = result.name
    for kk, vv in pairs(profarray[k]) do
     if (kk~="*") then
      result = me:LocalizeProf(kk,vv)
      if (spell[help]["more"]==nil) then spell[help]["more"] = {} end
      spell[help]["more"][result.name] = result
     end
    end
   end
  end
  return spell
end
--Set New Table
me.P.Primary = me:LocalizeList(me.P.Primary)
me.P.Secondary = me:LocalizeList(me.P.Secondary)
--Filter professions and add Ranks
function me:FilterProfs(profarray)                --Array from me:LocalizeList()
  local spell = {}
  for i = 1, GetNumSkillLines() do
   local skillName, isHeader, isExpanded, skillRank, _, _, skillMaxRank = GetSkillLineInfo(i)
   if (isHeader == nil) then
    if (profarray[skillName]~=nil) then
     spell[skillName] = profarray[skillName]
     spell[skillName]["name"] = nil
     spell[skillName]["rank"] = skillRank
     spell[skillName]["maxrank"] = skillMaxRank
    end
   elseif not isExpanded then
    ExpandSkillHeader(i)
   end
  end
  return spell	
end
--Get Weapon Skills
function me:GetWeaponSkills()
  local skills = {}
  local active = false
  for i = 1, GetNumSkillLines() do
   local skillName, isHeader, isExpanded, skillRank, _, _, skillMaxRank = GetSkillLineInfo(i)
   if (isHeader==1) then
    if (skillName==me.L["weapon"]) then
     active=true
     if not isExpanded then ExpandSkillHeader(i) end
    else
     active=false
    end
   else
    if (active == true) then
     skills[skillName] = {}
     skills[skillName]["rank"] = skillRank
     skills[skillName]["maxrank"] = skillMaxRank
    end
   end 
  end
  return skills
end



--------------------------------------------------------------------------------
--Table Alphabetical Sort Function                                            --
--------------------------------------------------------------------------------
function me:pairsByKeys (t, f)
  local a = {}
  for n in pairs(t) do table.insert(a, n) end
   table.sort(a, f)
   local i = 0                                    --iterator variable
   local iter = function ()                       --iterator function
   i = i + 1
   if a[i] == nil then
    return nil
   else
    return a[i], t[a[i]]
   end
  end
  return iter
end



--------------------------------------------------------------------------------
--Format Cooldown Duration                                                    --
--------------------------------------------------------------------------------
function me:formatduration(duration)
  local d,h,m
  d = floor(duration/86400)
  h = floor(duration/3600)-(d*24)
  m = floor(duration/60)-((d*1440)+(h*60))
  if d==0 then
   if h==0 then
    if m==0 then
     return "<1"..me.L["m"]
    else
     return m..me.L["m"]
    end
   else
    return h..me.L["h"].." "..m..me.L["m"]
   end
  else
   return d..me.L["d"].." "..h..me.L["h"]
  end 
end



--------------------------------------------------------------------------------
-- tcount: count table members even if they're not indexed by numbers         --
--------------------------------------------------------------------------------
function me:tcount(tab)
  local n = #tab
  if (n == 0) then
   for _ in pairs(tab) do
    n = n + 1
   end
  end
  return n
 end



--------------------------------------------------------------------------------
-- Load/Save Saved Variables
--------------------------------------------------------------------------------
function me.frame:OnEvent(event, arg1)
  if event == "ADDON_LOADED" and arg1 == "Broker_ProfessionsMenu" then
  ---Default values
   if (ProfessionsMenu == nil) then ProfessionsMenu = {} end
   if (ProfessionsMenu[GetCVar("realmName")] == nil) then ProfessionsMenu[GetCVar("realmName")] = {} end
   if (ProfessionsMenu[GetCVar("realmName")][UnitName("player")] == nil) then ProfessionsMenu[GetCVar("realmName")][UnitName("player")] = {} end
   --Copy saves to internal var and initializise
   me.save = ProfessionsMenu[GetCVar("realmName")][UnitName("player")]
   me.saveallchars = ProfessionsMenu[GetCVar("realmName")]  --all Chars from same server
   if (me.save.config == nil) then
    me.save.config = {
     showprimary = true,
     showsecondary = true,
     showother = true,
     showcds = true,
    }
   end
   if (me.save.default == nil) then
    me.save.default = {
     left = "---",
     shiftleft = "---",
     altleft = "---",
     ctrlleft = "---",
     icon = "Interface\\Icons\\Trade_BlackSmithing"
    }
   end
   if (me.save.cds == nil) then me.save.cds = {} end
   if (me.save.tradelinks == nil) then me.save.tradelinks = {} end
   if (me.save.craftableitems == nil) then me.save.craftableitems = {} end--NEW
   if (me.save.launcher == nil) then me.save.launcher = {} end
   me.save.faction = UnitFactionGroup("player")
   me.saveallchars[UnitName("player")] = me.save
  elseif event == "PLAYER_LOGIN" then
   --Init LibDataBroker
   me:InitLDB(me.save.default.left,me.save.default.icon)
   --Create Launchers 
   for k,v in pairs(me.save.launcher) do
    me.launcher[k]=me:newlauncher(k,v)
   end
   --Modifying Atlasloot Recept Tooltips
   if AtlasLootTooltip then
    AtlasLootTooltip:HookScript("OnTooltipSetSpell", function(self, ...)
     me:modifytooltip(self,select(3,self:GetSpell()))
    end)
   end
   --
  elseif event == "PLAYER_LOGOUT" then
   ProfessionsMenu[GetCVar("realmName")] = me.saveallchars
   ProfessionsMenu[GetCVar("realmName")][UnitName("player")] = me.save
  --[[Scan for recepts]]--
  elseif event == "TRADE_SKILL_SHOW" then --Scann for recepts
   if not IsTradeSkillLinked() and GetNumTradeSkills() and GetTradeSkillListLink() then
    local prof = strmatch(GetTradeSkillListLink(),"|h%[([%a%s]+)%]|h")
    me.save.craftableitems[prof] = ""
    --Remove deleted Profession
    local craftableitems = me.save.craftableitems
    me.save.craftableitems = {}
    for k,v in pairs(me:FilterProfs(me.P.Primary)) do
     if craftableitems[k]~=nil then me.save.craftableitems[k] = craftableitems[k] end
    end
    for k,v in pairs(me:FilterProfs(me.P.Secondary)) do
     if craftableitems[k]~=nil then me.save.craftableitems[k] = craftableitems[k] end
    end
    --
    for i = 1, GetNumTradeSkills() do
     local _,skilltype,_,isExpanded = GetTradeSkillInfo(i)
     if skilltype~="header" then
      local enchant=GetTradeSkillRecipeLink(i)
      local item=GetTradeSkillItemLink(i)
      if type(enchant)=="string" then
       enchant=strmatch(enchant,"|Henchant:(%d+)")
       if type(item)=="string" then
        item=strmatch(item,"|Hitem:(%d+):")
        if not item then item="0" end
       else
        item="0"
       end
        me.save.craftableitems[prof] = me.save.craftableitems[prof].."|"..enchant..","..item.."|"
      end
     else
      if not isExpanded then ExpandTradeSkillSubClass(i) end
     end
    end
   end
  --[[Scan professions for CDs]]--
  elseif event == "TRADE_SKILL_UPDATE" then
   if not IsTradeSkillLinked() and GetNumTradeSkills()~=nil then
    if GetTradeSkillListLink()~=nil then
     me.save.tradelinks[strmatch(GetTradeSkillListLink(),"|h%[([%a%s]+)%]|h")] = GetTradeSkillListLink() 
     --Remove deleted Profession
     local tradelinks = me.save.tradelinks
     me.save.tradelinks = {}
     for k,v in pairs(me:FilterProfs(me.P.Primary)) do
      if tradelinks[k]~=nil then
       me.save.tradelinks[k] = tradelinks[k]
      end
     end
     for k,v in pairs(me:FilterProfs(me.P.Secondary)) do
      if tradelinks[k]~=nil then
       me.save.tradelinks[k] = tradelinks[k]
      end
     end
     --
    end
    for i = 1, GetNumTradeSkills() do
     local name,skilltype = GetTradeSkillInfo(i)
     if skilltype~="header" then
      local cd=GetTradeSkillCooldown(i)
      if cd then
       if me.sharedcds[name] then name=me.sharedcds[name] end
       me.save.cds[name]=time()+floor(cd+0.5)
      end
     end
    end 
   end
  end
end
me.frame:SetScript("OnEvent", me.frame.OnEvent)   --Register Events



--------------------------------------------------------------------------------
-- Modifying Tooltips                                                         --
--------------------------------------------------------------------------------
--Tooltip Manipulation
function me:modifytooltip(tooltip,id,isitem)
  local chars,prof
  me.saveallchars[UnitName("player")]["craftableitems"]=me.save.craftableitems
  for k,v in pairs(me.saveallchars) do
   if v.craftableitems and v.faction==UnitFactionGroup("player") then
    for kk,vv in pairs(v.craftableitems) do
     local found
     if isitem then --Get ID of Henchant
      found = strfind(vv,"|%d+,"..id.."|") -- |ECHANT,ITEM|
     else
      found = strfind(vv,"|"..id..",%d+|")
     end
     if found then
      prof=kk
      if not chars then chars=k else chars=chars..", "..k end
     end
    end
   end
  end
  if prof then
   tooltip:AddLine(' ')
   tooltip:AddLine(prof..": "..chars,1,0.60,0)
   tooltip:Show()
  end
end
--Modifying Item Tooltips
me.TipHooker:Hook(function(self,...)
   local itemid = strmatch(select(2,self:GetItem()),"|Hitem:(%d+):")
   me:modifytooltip(self,itemid,true)
  end, "item")
--Modifying Enchant Recept Tooltips
hooksecurefunc(ItemRefTooltip, "SetHyperlink", function(self, ...)
 if select(1,strfind(arg1,"enchant:")) then
  local id=strmatch(arg1,"enchant:(%d+)")
  me:modifytooltip(self,id)
 end
end)



--------------------------------------------------------------------------------
-- Init LibDataBroker (Main Window)
--------------------------------------------------------------------------------
function me:InitLDB(txt,icon)
  --Create Secure Frame Overlay
  me.secureframe = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate");
  me.secureframe:SetScript("OnLeave",function(self)
   if not InCombatLockdown() then self:Hide() end
   GameTooltip:Hide()
  end)
  me.secureframe:SetScript("OnEnter",function(self) me:tooltip(self); GameTooltip:Show() end)
  me.secureframe:SetScript("OnMouseDown",function(self,button)
   me.secureframe:SetAttribute("*type1", "spell")
   if IsControlKeyDown() then
    me.secureframe:SetAttribute("spell", me.save.default.ctrlleft)
   elseif IsAltKeyDown() then
    me.secureframe:SetAttribute("spell", me.save.default.altleft)
   elseif IsShiftKeyDown() then
    me.secureframe:SetAttribute("spell", me.save.default.shiftleft)
   else
    me.secureframe:SetAttribute("spell", me.save.default.left)
   end
   if button == "RightButton" then
    GameTooltip:Hide()
    me.dewdrop:Open(self, 'children', function(level, value) me:UpdateMenu(level, value) end)
   end
  end)
  --Create LDB
  me.dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Broker_ProfessionsMenu", {
   type = "data source",
   text = txt,
   icon = icon,
   OnClick = function(self,btn)
    if InCombatLockdown() then
     me:Error(me.L["infight"])
     return
    end
    if btn == "RightButton" then
     me.moveable = false
     GameTooltip:Hide()
    end
   end,
   OnEnter = function(self)
    if InCombatLockdown() then
     return
    end
    if me.moveable then
     me:moveabletooltip(self)
     GameTooltip:Show()
     return
    end
    me.secureframe:SetFrameStrata(self:GetFrameStrata())
    me.secureframe:SetFrameLevel(self:GetFrameLevel()+1)
    me.secureframe:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT",0,0)
    me.secureframe:SetWidth(self:GetWidth())
    me.secureframe:SetHeight(self:GetHeight()) 
    me.secureframe:Show()
   end
  })
end



--------------------------------------------------------------------------------
-- Refresh broker icon/text
--------------------------------------------------------------------------------
function me:Professions_UpdateInfo(txt,icon)
  me.dataobj.text = txt
  me.dataobj.icon = icon
end



--------------------------------------------------------------------------------
-- Error Messages / Other Messages
--------------------------------------------------------------------------------
function me:Error(txt)
  UIErrorsFrame:AddMessage(txt,1.0,0,0,100,3);
end



--------------------------------------------------------------------------------
-- Creates Launchers
--------------------------------------------------------------------------------
function me:newlauncher(spell,icon)
  local launcher = {}
  --secure frame
  launcher.secureframe = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate")
  launcher.secureframe:SetScript("OnLeave",function(self)
   if not InCombatLockdown() then self:Hide() end
   GameTooltip:Hide()
  end)
  launcher.secureframe:SetScript("OnEnter",function(self)
   me:quicklaunchertooltip(self,spell)
   GameTooltip:Show()
  end)
  launcher.secureframe:SetAttribute("type1", "spell")
  launcher.secureframe:SetAttribute("spell", spell)
  --Create Dataobject
  launcher.dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Launcher: "..spell, {
   type = "data source",
   text = spell,
   icon = icon,
   OnClick = function(self,btn)
    if InCombatLockdown() then
     me:Error(me.L["infight"])
     return
    end
    if btn == "RightButton" then
     me.moveable = false
     GameTooltip:Hide()
    end
   end,
   OnEnter = function(self)
    if InCombatLockdown() then
     return
    end    
    if me.moveable then
     me:moveabletooltip(self)
     GameTooltip:Show()
     return 
    end
    launcher.secureframe:SetFrameStrata(self:GetFrameStrata())
    launcher.secureframe:SetFrameLevel(self:GetFrameLevel()+1)
    launcher.secureframe:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT",0,0)
    launcher.secureframe:SetWidth(self:GetWidth())
    launcher.secureframe:SetHeight(self:GetHeight()) 
    launcher.secureframe:Show()
   end
  })
  return launcher
end